/*
* Copyright 2009-2010, Andrew Barry
*
* some small improvements Copyright 2012, Hans Paijmans (paai)
*
* This file is part of MakerScanner.
*
* MakerScanner is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License (Version 2, June 1991) as published by
* the Free Software Foundation.
*
* MakerScanner is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ActiveStereoApp.h"

//(*AppHeaders
#include "ActiveStereoMain.h"
#include <wx/image.h>
//*)

IMPLEMENT_APP(ActiveStereoApp);

bool ActiveStereoApp::OnInit()
{

  // mandatory to read commandline parms (paai)

  version=wxT("0.4");

    if (!wxApp::OnInit())
        return false;

    //(*AppInitialize
    bool wxsOK = true;    


    wxInitAllImageHandlers();
    if ( wxsOK )
    {
    	ActiveStereoFrame* Frame = new ActiveStereoFrame(0);
    	Frame->Show();
    	SetTopWindow(Frame);
    }

    //*)
    return wxsOK;

}

void ActiveStereoApp::OnInitCmdLine(wxCmdLineParser& parser)
{
    parser.SetDesc (g_cmdLineDesc);
    // must refuse '/' as parameter starter or cannot use "/path" style paths
    parser.SetSwitchChars (wxT("-"));
}
 
bool ActiveStereoApp::OnCmdLineParsed(wxCmdLineParser& parser)
{
  wxString holder;
  //  double dist;

    DISPARITY_DISTANCE=19.8;
    thresholdPixelValue = 20;
    brightnessFilterValue = 0.25;
    pedestal_height=450;
    rainbow_factor=200;
    background_depth=25;

    verbose =       parser.Found(wxT("v"));
    paai_help =     parser.Found(wxT("h"));
    debug =         parser.Found(wxT("D"));
    no_background=  parser.Found(wxT("n"));

    if (parser.Found(_T("N"), &holder))
      {
      holder.ToLong(&background_depth);
      if (!background_depth) background_depth=25;
      no_background=1;
      }


    if (parser.Found(_T("d"), &holder))
      {
      holder.ToDouble(&DISPARITY_DISTANCE);
      }

    if (parser.Found(_T("r"), &holder))
      {
      holder.ToLong(&rainbow_factor);
      }

    if (parser.Found(_T("t"), &holder))
      {
      holder.ToLong(&thresholdPixelValue);
      }

    if (parser.Found(_T("p"), &holder))
      {
       holder.ToLong(&pedestal_height);
      }

    if (parser.Found(_T("b"), &holder))
      {
       holder.ToDouble(&brightnessFilterValue);
      }


    if (paai_help)
      {      
        printf("\nVersion: %s\n",version.mb_str().data());

	cout << "Usage: makerscanner [options] [pointcloudfile (ply or obj)]\n";
	cout << "  -h    : this help\n";
        cout << "  -D    : debug\n";
        cout << "  -n    : suppress background\n";
        cout << "  -d #  : distance between laser and camera (default 19.8)\n";
        cout << "  -t #  : Brightness Threshold (2 - 50; default ";
	    cout << thresholdPixelValue;
            cout << ")\n";

        cout << "  -b #  : Brightness Filter (0 - 1.0; default ";
	    cout << brightnessFilterValue;
            cout << ")\n";
        cout << "  -p #  : Pedestal height, below with no data will be saved. Default bottom of image\n";
        cout << "  -r #  : rainbow shading for depth (default ";
            cout << rainbow_factor;
            cout << ")\n";

        cout << "\nIf pointcloudfile is given on the commandline, the file dialog window is suppressed.\n";
        cout << "Clicking \"Camera Connect\" will reset the filename and re-enable the file dialog window.\n";

        cout << "\nBrightness Threshold controls how bright the laser must be in the image \n";
        cout << "for it to be detected.  Low values will be more forgiving of light lasers or \n";
        cout << "dark objects, but tend to produce more noise. \n\n";
        cout << "Brightness Filter controls how bright pixels must appear relative to other \n";
        cout << "laser pixels.  Low values are more forgiving but tend to produce more noise.\n";

        cout << "\nThe pointcloud file and thresholdvalues can be adjusted later in the GUI.\n\n";

        cout << "The top 25 pixels (above the green line) are reserved for a flat surface. \n";
        cout << "If the entire top of the image is not looking at a flat surface, the scan \n";
        cout << "will not work well.\n";
        cout << "Rainbow shading shows depth in the scan window. Low values (100-300)for small objects.\n";
        cout << "Try not to move the camera or objects during the scan.  The system uses an \n";
        cout << "image-difference technique which means that it is sensitive to any changes in\n";
        cout << " the image.\n\n";
        return false;
      }

    // cout << silent_mode;

    // to get at your unnamed parameters use
    wxArrayString files;
    for (int i = 0; i < parser.GetParamCount(); i++)
        {
        files.Add(parser.GetParam(i));
        if (i==0) paai_filename << parser.GetParam(i);
        }
 	    
    if ((verbose) || (debug))
      {
      cout << "\nDISPARITY_DISTANCE    : ";      cout << DISPARITY_DISTANCE;
      cout << "\nthresholdPixelValue   : ";      cout << thresholdPixelValue;
      cout << "\nbrightnessFilterValue : ";      cout << brightnessFilterValue;
      cout << "\nno_background         : ";      cout << no_background;
      cout << "\nrainbow factor        : ";      cout << rainbow_factor;
      cout << "\nbackground_depth      : ";      cout << background_depth;


      printf("\nfilenaam: %s\n",paai_filename.mb_str().data());

      // exit (0);
      }
   
    // and other command line parameters
 
    // then do what you need with them.

    return true;
}
